@@ -1,5 +1,8 @@
+import math
 from typing import Union
+
 import torch
+import torch.nn.functional as F
 from torch import nn
 
 from mcquic import Consts
@@ -85,84 +88,46 @@ class _logExpMinusOne(torch.autograd.Function):
     @staticmethod
     def forward(ctx, x, bound):
         ctx.save_for_backward(x, bound)
-        return (x.exp() - 1 + Consts.Eps).log()
+        return (x.exp() - 1 + torch.finfo(x.dtype).eps).log()
 
     @staticmethod
     def backward(ctx, grad_output):
         x, bound = ctx.saved_tensors
         passThroughIf = x > bound
         remaining = ~passThroughIf
-        return passThroughIf * grad_output + remaining * grad_output * x.exp() / (x.exp() - 1 + Consts.Eps), None
+        return passThroughIf * grad_output + remaining * grad_output * x.exp() / (x.exp() - 1 + torch.finfo(x.dtype).eps), None
 
 class LogExpMinusOne(nn.Module):
-    def __init__(self, eps: Union[torch.Tensor, float] = Consts.Eps):
+    def __init__(self):
         super().__init__()
-        eps = torch.tensor(eps, dtype=torch.float)
+        eps = torch.tensor(torch.finfo(torch.float).eps)
         self.register_buffer("_bound", ((1 + eps) / eps).log())
 
     def forward(self, x: torch.Tensor) -> torch.Tensor:
         return _logExpMinusOne.apply(x, self._bound)
 
-def logExpMinusOne(x: torch.Tensor, eps: Union[torch.Tensor, float] = Consts.Eps) -> torch.Tensor:
-    eps = torch.tensor(eps, dtype=torch.float, device=x.device)
+def logExpMinusOne(x: torch.Tensor) -> torch.Tensor:
+    eps = torch.tensor(torch.finfo(x.dtype).eps, device=x.device)
     return _logExpMinusOne.apply(x, ((1 + eps) / eps).log())
 
 
-def gumbelArgmaxRandomPerturb(logits: torch.Tensor, perturbRate: float = 0.0, tau: float = 1, dim: int = -1) -> torch.Tensor:
-    r"""
-    Samples from the Gumbel-Argmax distribution and optionally perturb (modify sample to another result).
-
-    Args:
-      logits: `[..., num_features]` unnormalized log probabilities
-      perturbRate: how many samples needs perturbation
-      tau: non-negative scalar temperature
-      dim (int): A dimension along which softmax will be computed. Default: -1.
-
-    Returns:
-      Sampled tensor of same shape as `logits` from the Gumbel-Softmax distribution.
-      If ``hard=True``, the returned samples will be one-hot, otherwise they will
-      be probability distributions that sum to 1 across `dim`.
-
-    .. note::
-      This function is here for legacy reasons, may be removed from nn.Functional in the future.
-
-    .. note::
-      The main trick for `hard` is to do  `y_hard - y_soft.detach() + y_soft`
-
-      It achieves two things:
-      - makes the output value exactly one-hot
-      (since we add then subtract y_soft value)
-      - makes the gradient equal to y_soft gradient
-      (since we strip all other gradients)
-
-    Examples::
-        >>> logits = torch.randn(20, 32)
-        >>> # Sample soft categorical using reparametrization trick:
-        >>> F.gumbel_softmax(logits, tau=1, hard=False)
-        >>> # Sample hard categorical using "Straight-through" trick:
-        >>> F.gumbel_softmax(logits, tau=1, hard=True)
-    """
-    gumbels = (
-        -torch.empty_like(logits, memory_format=torch.legacy_contiguous_format).exponential_().log()
-    )  # ~Gumbel(0,1)
-    gumbels = (logits + gumbels) / tau  # ~Gumbel(logits,tau)
-    y_soft = gumbels.softmax(dim)
-
-    # Straight through.
-    index = y_soft.max(dim, keepdim=True)[1]
-
-    perturbCount = round(index.numel() * perturbRate)
-    if perturbCount > 0:
-        newIdx = torch.randint(logits.shape[dim], size=(perturbCount, ), device=index.device, dtype=index.dtype)
-
-        # inplace opertaion
-        continguous = index.view(-1)
-        permIdx = torch.randperm(len(continguous))
-        continguous[permIdx[:perturbCount]] = newIdx
-
-    y_hard = torch.zeros_like(logits, memory_format=torch.legacy_contiguous_format).scatter_(dim, index, 1.0)
-    ret = y_hard - y_soft.detach() + y_soft
-    return ret
-
 def oneHot(x: torch.LongTensor, numClasses: int, dim: int = -1, dtype = torch.float):
     return torch.zeros((*x.shape, numClasses), dtype=dtype).scatter_(dim, x, 1)
+
+
+def gumbelSoftmax(logits: torch.Tensor, temperature: float = 1.0, hard: bool = True, dim: int = -1):
+    eps = torch.finfo(logits.dtype).eps
+    uniforms = torch.rand_like(logits).clamp_(eps, 1 - eps)
+    gumbels = -((-(uniforms.log())).log())
+
+    y_soft = ((logits + gumbels) / temperature).softmax(dim)
+
+    if hard:
+        # Straight through.
+        index = y_soft.max(dim, keepdim=True)[1]
+        y_hard = torch.zeros_like(logits, memory_format=torch.legacy_contiguous_format).scatter_(dim, index, 1.0)
+        ret = y_hard - y_soft.detach() + y_soft
+    else:
+        # Reparametrization trick.
+        ret = y_soft
+    return ret
