@@ -1,4 +1,4 @@
-from typing import Callable, Tuple, Type
+from typing import Type
 
 from vlutils.base import Registry
 
@@ -26,3 +26,6 @@ class OptimizerRegistry(Registry[Type["torch.optim.optimizer.Optimizer"]]):
 
 class LossRegistry(Registry[Type["torch.nn.Module"]]):
     pass
+
+class HookRegistry(Registry[Type["mcquic.train.hooks.Hook"]]):
+    pass
